@@ -7,9 +7,9 @@ from typing import List, Tuple, Union
 
 import pandas as pd
 import structlog
-from openstef.data_classes.prediction_job import PredictionJobDataClass
 
 from openstef.data_classes.model_specifications import ModelSpecificationDataClass
+from openstef.data_classes.prediction_job import PredictionJobDataClass
 from openstef.exceptions import (
     InputDataInsufficientError,
     InputDataWrongColumnOrderError,
@@ -35,38 +35,44 @@ def train_model_pipeline(
     pj: PredictionJobDataClass,
     input_data: pd.DataFrame,
     check_old_model_age: bool,
-    trained_models_folder: Union[str, Path],
-) -> None:
-    """Midle level pipeline that takes care of all persistent storage dependencies
+    mlflow_tracking_uri: str,
+    artifact_folder: str,
+) -> Report:
+    """Middle level pipeline that takes care of all persistent storage dependencies
 
-    Expected prediction jobs keys: "id", "model", "hyper_params",
-        "feature_names"
+    Expected prediction jobs keys: "id", "model", "hyper_params", "feature_names"
 
     Args:
         pj (PredictionJobDataClass): Prediction job
         input_data (pd.DataFrame): Raw training input data
         check_old_model_age (bool): Check if training should be skipped because the model is too young
-        trained_models_folder (Path): Path where trained models are stored
+        mlflow_tracking_uri (str): Tracking URI for MLFlow
+        artifact_folder (str): Path where artifacts, such as trained models, are stored
 
     Returns:
-        None
-
+        report (Report): The report containing train/val/test datasets and corresponding forecasts if requested
     """
-    # Intitialize logger and serializer
+    # Initialize logger and serializer
     logger = structlog.get_logger(__name__)
-    serializer = MLflowSerializer(trained_models_folder)
+    serializer = MLflowSerializer(mlflow_tracking_uri=mlflow_tracking_uri)
 
     # Get old model and age
     try:
-        old_model, modelspecs = serializer.load_model(pj["id"])
-        old_model_age = (
-            old_model.age
-        )  # Age attribute is openstef specific and is added by the serializer
+        old_model, model_specs = serializer.load_model(experiment_name=str(pj["id"]))
+        old_model_age = old_model.age  # Age attribute is openstef specific
     except (AttributeError, FileNotFoundError, LookupError):
         old_model = None
         old_model_age = float("inf")
-        # create basic modelspecs
-        modelspecs = ModelSpecificationDataClass(id=pj["id"])
+        if pj["default_modelspecs"] is not None:
+            model_specs = pj["default_modelspecs"]
+            if model_specs.id != pj.id:
+                raise RuntimeError(
+                    "The id of the prediction job and its default model_specs do not"
+                    " match."
+                )
+        else:
+            # create basic model_specs
+            model_specs = ModelSpecificationDataClass(id=pj["id"])
         logger.warning("No old model found, training new model", pid=pj["id"])
 
     # Check old model age and continue yes/no
@@ -78,8 +84,8 @@ def train_model_pipeline(
 
     # Train model with core pipeline
     try:
-        model, report, modelspecs_updated = train_model_pipeline_core(
-            pj, modelspecs, input_data, old_model
+        model, report, model_specs_updated = train_model_pipeline_core(
+            pj, model_specs, input_data, old_model, horizons=pj.train_horizons_minutes
         )
     except OldModelHigherScoreError as OMHSE:
         logger.error("Old model is better than new model", pid=pj["id"], exc_info=OMHSE)
@@ -95,25 +101,37 @@ def train_model_pipeline(
 
     except InputDataWrongColumnOrderError as IDWCOE:
         logger.error(
-            "Wrong column order, 'load' column should be first and 'horizon' column last.",
+            "Wrong column order, 'load' column should be first and 'horizon' column"
+            " last.",
             pid=pj["id"],
             exc_info=IDWCOE,
         )
         raise InputDataWrongColumnOrderError(IDWCOE)
 
-    # Save model
-    serializer.save_model(model, pj=pj, modelspecs=modelspecs_updated, report=report)
+    # Save model and report. Report is always saved to MLFlow and optionally to disk
+    serializer.save_model(
+        model=model,
+        experiment_name=str(pj["id"]),
+        model_type=pj["model"],
+        model_specs=model_specs_updated,
+        report=report,
+    )
+    if artifact_folder:
+        Reporter.write_report_to_disk(report=report, artifact_folder=artifact_folder)
 
     # Clean up older models
-    serializer.remove_old_models(pj=pj)
+    serializer.remove_old_models(
+        experiment_name=str(pj["id"]), artifact_folder=artifact_folder
+    )
+    return report
 
 
 def train_model_pipeline_core(
     pj: PredictionJobDataClass,
-    modelspecs: ModelSpecificationDataClass,
+    model_specs: ModelSpecificationDataClass,
     input_data: pd.DataFrame,
     old_model: OpenstfRegressor = None,
-    horizons: List[float] = None,
+    horizons: Union[List[float], str] = None,
 ) -> Tuple[OpenstfRegressor, Report, ModelSpecificationDataClass]:
     """Train model core pipeline.
     Trains a new model given a prediction job, input data and compares it to an old model.
@@ -121,7 +139,7 @@ def train_model_pipeline_core(
 
     Args:
         pj (PredictionJobDataClass): Prediction job
-        modelspecs (ModelSpecificationDataClass): Dataclass containing model specifications
+        model_specs (ModelSpecificationDataClass): Dataclass containing model specifications
         input_data (pd.DataFrame): Input data
         old_model (OpenstfRegressor, optional): Old model to compare to. Defaults to None.
         horizons (List[float]): horizons to train on in hours.
@@ -136,15 +154,18 @@ def train_model_pipeline_core(
     """
 
     if horizons is None:
-        horizons = DEFAULT_TRAIN_HORIZONS
+        if pj.train_horizons_minutes is None:
+            horizons = DEFAULT_TRAIN_HORIZONS
+        else:
+            horizons = pj.train_horizons_minutes
 
     logger = structlog.get_logger(__name__)
 
     # Call common pipeline
     model, report, train_data, validation_data, test_data = train_pipeline_common(
-        pj, modelspecs, input_data, horizons
+        pj, model_specs, input_data, horizons
     )
-    modelspecs.feature_names = list(train_data.columns)
+    model_specs.feature_names = list(train_data.columns)
 
     # Check if new model is better than old model
     if old_model:
@@ -175,14 +196,14 @@ def train_model_pipeline_core(
         except ValueError as e:
             logger.info("Could not compare to old model", pid=pj["id"], exc_info=e)
 
-    return model, report, modelspecs
+    return model, report, model_specs
 
 
 def train_pipeline_common(
     pj: PredictionJobDataClass,
-    modelspecs: ModelSpecificationDataClass,
+    model_specs: ModelSpecificationDataClass,
     input_data: pd.DataFrame,
-    horizons: List[float],
+    horizons: Union[List[float], str],
     test_fraction: float = 0.0,
     backtest: bool = False,
     test_data_predefined: pd.DataFrame = pd.DataFrame(),
@@ -191,7 +212,7 @@ def train_pipeline_common(
 
     Args:
         pj (PredictionJobDataClass): Prediction job
-        modelspecs (ModelSpecificationDataClass): Dataclass containing model specifications
+        model_specs (ModelSpecificationDataClass): Dataclass containing model specifications
         input_data (pd.DataFrame): Input data
         horizons (List[float]): horizons to train on in hours.
         test_fraction (float): fraction of data to use for testing
@@ -215,10 +236,23 @@ def train_pipeline_common(
             "Missing the load column in the input dataframe"
         )
 
+    if isinstance(horizons, str):
+        if not (horizons in set(input_data.columns)):
+            raise ValueError(
+                f"The horizon parameter specifies a column name ({horizons}) missing in"
+                " the input data."
+            )
+        else:
+            # sort data to avoid same date repeated multiple time
+            input_data = input_data.sort_values(horizons)
     # Validate and clean data
-    validated_data = validation.clean(validation.validate(pj["id"], input_data))
+    validated_data = validation.drop_target_na(
+        validation.validate(pj["id"], input_data, pj["flatliner_treshold"])
+    )
     # Check if sufficient data is left after cleaning
-    if not validation.is_data_sufficient(validated_data):
+    if not validation.is_data_sufficient(
+        validated_data, pj["completeness_treshold"], pj["minimal_table_length"]
+    ):
         raise InputDataInsufficientError(
             "Input data is insufficient, after validation and cleaning"
         )
@@ -232,8 +266,8 @@ def train_pipeline_common(
 
     data_with_features = TrainFeatureApplicator(
         horizons=horizons,
-        feature_names=modelspecs.feature_names,
-        feature_modules=modelspecs.feature_modules,
+        feature_names=model_specs.feature_names,
+        feature_modules=model_specs.feature_modules,
     ).add_features(validated_data, pj=pj)
 
     # if test_data is predefined, apply the pipeline only on the remaining data
@@ -285,7 +319,7 @@ def train_pipeline_common(
     protected_hyperparams = ["quantiles"]
     valid_hyper_parameters = {
         key: value
-        for key, value in modelspecs.hyper_params.items()
+        for key, value in model_specs.hyper_params.items()
         if key in model.get_params().keys() and key not in protected_hyperparams
     }
 
@@ -307,6 +341,11 @@ def train_pipeline_common(
         validation_data
     ).generate_standard_deviation_data(model)
 
+    if pj.save_train_forecasts:
+        train_data["forecast"] = model.predict(train_data).values
+        validation_data["forecast"] = model.predict(validation_data).values
+        test_data["forecast"] = model.predict(test_data).values
+
     # Report about the training process
     reporter = Reporter(train_data, validation_data, test_data)
     report = reporter.generate_report(model)
